<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

/**
 * 	This is the main entry point for the Woogle4MediaWiki extension
 * 
 *  Write
 *  	require_once( "$IP/extensions/Woogle/Woogle.php" );
 *  into your LocalSettings.php to activate Woogle for your MediaWiki
 *
 *  Thus, this file as such contains mainly code to intialize/setup the 
 *  Woogle Extension within its surrounding MediaWiki instance
 */

# Not a valid entry point, skip unless MEDIAWIKI is defined
if (!defined('MEDIAWIKI')) {
        echo <<<EOT
To install my extension, put the following line in LocalSettings.php:
require_once( "$IP/extensions/Woogle/Woogle.php" );
EOT;
        exit( 1 );	// exit with error code
}

// include base files
require_once(dirname(__FILE__) . '/includes/WoogleConfig.php');		// main configuration
require_once(dirname(__FILE__) . '/includes/WoogleUtil.php');		// utility classes
require_once(dirname(__FILE__) . '/includes/WoogleFunctions.php');	// some initialization helper functions
require_once(dirname(__FILE__) . '/includes/WoogleHooks.php');		// contains callback functions for MediaWiki hooks

define('W4M_VERSION','1.0');											// base version for math. comparisons
define('W4M_VERSION_LONG', W4M_VERSION . '-RC2 ($Rev: 3844 $)');		// full version info for output
if (!defined('NS_FILE')) define('NS_FILE', NS_IMAGE);					// NS_FILE is not defined for MW <= 1.14.0

// register additional code for PHP autoloading (c.f. http://php.net/autoload)
// add Woogle core classes here - but add addon classes in addon initialization!
woogleAutoloadClasses(array(
	'WoogleArticleSearch'		=> '/includes',
	'WoogleFd'					=> '/includes',
	'WoogleLang'				=> '/includes',
	'WoogleQueryManager'		=> '/includes',
	'WoogleIndexService'		=> '/includes',
	'WoogleQueryService'		=> '/includes',
	'WoogleQueryServiceCommon'	=> '/includes',
	'WoogleRedLinks'			=> '/includes',
	'WoogleSearch'				=> '/includes',
	'WoogleMWUtil'				=> '/includes',
	'WoogleAlertManager'		=> '/includes',
	'WoogleJob'					=> '/includes/jobs',
	'WoogleJobAddClick'			=> '/includes/jobs',
	'WoogleJobRefreshItem'		=> '/includes/jobs',
	'WoogleJobTimeout'			=> '/includes/jobs',
	'WoogleSpecialConfig'		=> '/includes/specialpages',
	'WoogleSpecialFeedback'		=> '/includes/specialpages',
	'WoogleSpecialSearch'		=> '/includes/specialpages',
	'WoogleSpecialDummySearch'	=> '/includes/specialpages',
	'WoogleSpecialStatistics'	=> '/includes/specialpages',
	'WoogleSpecialResultInfo'	=> '/includes/specialpages',
	'WoogleSpecialHelp'			=> '/includes/specialpages',
	'WoogleEntityFreq'			=> '/includes/data',
	'WoogleTermFreq'			=> '/includes/data',
	'WoogleDocument'			=> '/includes/data',
	'WooglePDFExtractor'		=> '/includes/extractors',
));

// maps MediaWiki Job API jobs to its handling classes (http://www.mediawiki.org/wiki/Manual:$wgJobClasses)
$wgJobClasses['woogleAddClick']		= 'WoogleJobAddClick';
$wgJobClasses['woogleRefreshItem']	= 'WoogleJobRefreshItem';
$wgJobClasses['woogleTimeout']		= 'WoogleJobTimeout';

// register Ajax functions (http://www.mediawiki.org/wiki/Manual:$wgAjaxExportList)
$wgAjaxExportList[] = 'woogleAjaxRedLinkText';
$wgAjaxExportList[] = 'woogleAjaxRedLinkVisit';
$wgAjaxExportList[] = 'woogleAjaxAutocomplete';
$wgAjaxExportList[] = 'woogleAjaxAutocompleteRedirect';					
$wgAjaxExportList[] = 'wooglePromoteResult';   
$wgAjaxExportList[] = 'woogleDegradeResult';
$wgAjaxExportList[] = 'woogleRemoveResult';
$wgAjaxExportList[] = 'woogleRestoreResult';
$wgAjaxExportList[] = 'woogleAddResult';
$wgAjaxExportList[] = 'woogleAddComment';
$wgAjaxExportList[] = 'woogleSaveInPlace';
$wgAjaxExportList[] = 'woogleAjaxAutocompleteAddResult';


// Woogle credits shown on page Special:Version
$wgExtensionCredits['other'][] = array(
	'name'			=> 'Woogle4MediaWiki',
	'description'	=> 'Social Search for MediaWiki',
	'version'		=> W4M_VERSION_LONG,
	'author'		=> 'Ben Romberg, [http://www.hjhappel.de Hans-J&ouml;rg Happel], Christian R&ouml;hr and S&ouml;ren Schlegel, [http://www.fzi.de/ipe FZI Karlsruhe], Germany',
	'url'			=> 'http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki',
);

// set encoding
if (mb_internal_encoding() != "UTF-8"){
	WoogleUtil::debug("Woogle.php: Setting mb_internal_encoding to UTF-8");
	if (!mb_internal_encoding("UTF-8")) WoogleUtil::error("Woogle.php: Can not set encoding to UTF-8!");
}

// initialize namespaces
woogleInitNamespaces();

// load optional Woogle addons (at least one index addon needs to be present)
woogleLoadAddons();

// load MediaWiki Special:SpecialPages defined by Woogle
// load after addons as they might have own special pages added with WoogleUtil::addSpecialPage
woogleLoadSpecialPages();

