<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

set_include_path(dirname(__FILE__) . '/hessian/');
require_once(dirname(__FILE__) . '/hessian/HessianClient.php');

/**
 * This is the remote implementation of WoogleQueryService, retrieving results
 * from a Java/Lucene-based TeamWeaverIS backend
 * (c.f. http://www.teamweaver.org/wiki/index.php/Integrated_Search)
 *
 * Most method calls thus map methods in the Java interface QueryService of
 * TeamWeaverIS
 * 
 * TeamWeaverIS offers Hessian-based webservices (http://hessian.caucho.com/)
 * which this class accessed with the PHP library HessianPHP
 */
class WoogleQueryServiceRemote extends WoogleQueryService {
	protected $proxy;
	protected $proxyUpdate;
	
	/**
	 *	Constructor initializes proxy object to access web services 
	 */
	public function __construct() {
		WoogleUtil::profileIn(__METHOD__);
		
		$serviceUrl = WoogleConfig::get('remoteQueryServiceUrl');
		if ($serviceUrl == '') $serviceUrl =  WoogleConfig::get('remoteBaseServiceUrl') . 'phpQuery';
		$this->proxy = &new HessianClient($serviceUrl, WoogleConfig::get('remoteOptions'));
		
		$userviceUrl = WoogleConfig::get('remoteUpdateServiceUrl');
		if ($userviceUrl == '') $userviceUrl =  WoogleConfig::get('remoteBaseServiceUrl') . 'phpUpdate';
		$this->proxyUpdate = &new HessianClient($userviceUrl, WoogleConfig::get('remoteOptions'));
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	
	public function checkQueryService(){
		WoogleUtil::profileIn(__METHOD__);
		// TODO change
		$r = 0;
		$challenge = 'ok';
		$response = 'returning' . $challenge;
		$echo = $this->proxy->echo($challenge);
		if ($echo == $response) $r = 999;
		return WoogleUtil::profileOut(__METHOD__, $r);
	}
	

	public function isNative(){
		return false;
	}
	
	public function getSortKeys() {
		WoogleUtil::profileIn(__METHOD__);
		$sortKeys = $this->proxy->getSortKeys();
		WoogleUtil::profileOut(__METHOD__);
		return $sortKeys;
	}
	
	public function doAddClick($searchString, $url, $position, $session) {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::debug('Adding click for URL: ' . $url . ' / query: ' . $searchString);
		try{
			$this->proxyUpdate->addClick($this->getServiceParams(), $position, $url, $searchString);
		} catch (Exception $e) {
			WoogleUtil::error("Could not log click for " . $url . " after  " . $searchString);
		}
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function getAutocomplete($autocomplete, $num) {
		WoogleUtil::profileIn(__METHOD__);
		$qStart = microtime(true);
		$res = array();
		$values = $this->proxy->getAutoComplete($autocomplete, 10);
		
		foreach ($values as $tf) {
			$tf['entity'] = utf8_encode($tf['entity']);
			array_push($res, $tf);
		}
		
		// do some logging
		$qStop = microtime(true);
		$qTime = $qStop - $qStart;
		WoogleUtil::perflog($qTime . ';' . 'autocomplete', 'query');
		WoogleUtil::profileOut(__METHOD__);
		return $res;
	}
	
	public function getAttributeKeys() {
		WoogleUtil::profileIn(__METHOD__);
		$keys = $this->proxy->getAttributeKeys();
		WoogleUtil::profileOut(__METHOD__);
		return $keys;
	}
	
	public function getAttributeValues($key) {
		WoogleUtil::profileIn(__METHOD__);
		$values;
		if ($key == "mimetype"){
			$values = $this->proxy->getAttributeValues($this->getServiceParams(), $key);
		} else {
			$values = $this->proxy->getAttributeValues($this->getServiceParams(), $key);
		}
		WoogleUtil::profileOut(__METHOD__);
		return $values;
	}
	
	public function executeKeywordQueryValues($searchString, $count, $start, $values, $referer='') {
		WoogleUtil::profileIn(__METHOD__);
		
		//
		// construct query object sent to the backend
		//
		
		$query = array();
		$query['searchString'] 						= utf8_decode($searchString);
		$query['refinementsMax'] 					= 8;
		$query['refinementsWeightFactor'] 			= 0.125;
		$query['contentMaxFragments'] 				= 3;
		$query['contentHighlightMinWeightFactor'] 	= 0.5;
		$query['sources'] 							= $values['sources'];
		$query['repositories'] 						= $values['repositories'];
		$query['mimetypes'] 						= $values['mimetypes'];
		$query['changedMin'] 						= $values['changedMin'] > 0 ? new SmallDateTime($values['changedMin']) : null;
		$query['changedMax'] 						= $values['changedMax'] > 0 ? new SmallDateTime($values['changedMax']) : null;
		$query['pagesMin'] 							= $values['pagesMin'];
		$query['pagesMax'] 							= $values['pagesMax'];
		$query['sizeMin'] 							= $values['sizeMin'];
		$query['sizeMax'] 							= $values['sizeMax'];
		$query['count'] 							= $count;
		$query['start'] 							= $start;
		$query['sortType'] 							= $values['sortType'];
		$query['sortDesc'] 							= $values['sortDesc'];
		$query['excludeId']							= $values['excludeId'] > 0 ? WoogleQueryServiceCommon::getId($values['excludeId']) : '';
		//print_r($query);
		
		//
		// execute query
		//
		
		// returns back resultList object/array with including result objects
		$resultList2 = $this->proxy->executeKeywordQuery($this->getServiceParams($referer), $query);
		// own result array which will be returned by this method later
		$result = array('numHits' => $resultList2['numHits'], 'refinements' => $resultList2['queryInfo']['refinements']);
		
		//
		// iterate results
		//
		
		$i = 0;
		foreach ($resultList2['results'] as $ro) {
			
			// Debug code
			WoogleUtil::debug('Label1: ' 	. $ro['label']);
			WoogleUtil::debug('Label2: ' 	. utf8_encode($ro['label']));
			WoogleUtil::debug('Label3: ' 	. utf8_decode($ro['label']));
			WoogleUtil::debug('Url1: ' 		. $ro['uri']);
			WoogleUtil::debug('Url2: ' 		. utf8_encode($ro['uri']));
			
			$result['results'][$i] = array();
			$result['results'][$i]['uri'] 		= utf8_encode($ro['uri']);
			$result['results'][$i]['label'] 	= utf8_encode($ro['label']);   // disabled due to http://octopus13.fzi.de:8080/browse/WOO-253
			//$result['results'][$i]['label'] 	= $ro['label'];
			$result['results'][$i]['mimetype'] 	= $ro['mimetype'];
			//$result['results'][$i]['mimetype'] = $ro['mimetype']['primaryType'] . '/' . $ro['mimetype']['subType'];
			$result['results'][$i]['source'] 	= $ro['source'];
			$result['results'][$i]['size'] 		= $ro['size'];
			$sizeChars = -1;
			if (isset($ro['sizeChars'])); $sizeChars = $ro['sizeChars'];
			$sizeWords = -1;
			if (isset($ro['sizeWords'])); $sizeWords = $ro['sizeWords'];
			$sizeLines = -1;
			if (isset($ro['sizeLines'])); $sizeLines = $ro['sizeLines'];
			$result['results'][$i]['sizeChars'] = $sizeChars;
			$result['results'][$i]['sizeWords'] = $sizeWords;
			$result['results'][$i]['sizeLines'] = $sizeLines;
			$result['results'][$i]['pages']		= $ro['pages'];
			$result['results'][$i]['relevance'] = $ro['relevance'];
			$result['results'][$i]['genre'] 	= $ro['genre'];
			$result['results'][$i]['scope'] 	= $ro['scope'];
			$result['results'][$i]['language'] 	= $ro['language'];
			$result['results'][$i]['created'] 	= WoogleMWUtil::getWoogleTimestamp( $ro['created'] );
			$result['results'][$i]['changed'] 	= WoogleMWUtil::getWoogleTimestamp( $ro['changed'] );
			
			//
			// get result snippets
			//
			$result['results'][$i]['content'] = '';
			for ($j = 0; $j < count($ro['contentList']); $j++) {
				
				// add separator after first fragment
				if ($j > 0) {
					$result['results'][$i]['content'] .= WoogleQueryService::$result_separator;
				}
				
				// iterate fragments of individual result snippets
				foreach ($ro['contentList'][$j] as $cf) {
					
					$cfx = $cf['text'];
					
					// Debug
					WoogleUtil::debug('Content1: ' . $cfx);
					WoogleUtil::debug('Content2: ' . utf8_encode($cfx));
					WoogleUtil::debug('Content3: ' . utf8_decode($cfx));
					
					// TODO: quickhack for: http://octopus13.fzi.de:8080/browse/WOO-237
					//$cfx = utf8_decode($cf['text']);
					
					// TODO: quickhack for: http://octopus13.fzi.de:8080/browse/WOO-264
					//$cfx = utf8_encode($cf['text']);
					
					$tox = min(300, strlen($cfx));
					$cfx = substr($cfx, 0, $tox);
					if ($tox == 300) $cfx .= "...";
					
					$fragment = utf8_encode($cfx);
					// for a highlighted fragment (e.g. matching query terms) add highlighting markup
					if ($cf['highlighted']) $fragment = WoogleQueryService::highlight($fragment);
					$result['results'][$i]['content'] .= $fragment;
				}
			}

			$i++;
		}
		WoogleUtil::profileOut(__METHOD__);
		return $result;
	}
	


	public function supportPages() {
		WoogleUtil::profileIn(__METHOD__);
		// TODO ??
		return WoogleUtil::profileOut(__METHOD__, true);
	}
	

	
	private function getValues($valueList) {
		WoogleUtil::profileIn(__METHOD__);
		if (is_array($valueList)) {
			$value = "";
			foreach ($valueList as $v) {
				if ($value != "") {
					$value .= " ... ";
				}
				$value .= $v;
			}
			WoogleUtil::profileOut(__METHOD__);
			return $value;
		}
		else {
			WoogleUtil::profileOut(__METHOD__);
			return $valueList;
		}
	}
	
	private function getServiceParams($referer='') {
		WoogleUtil::profileIn(__METHOD__);
		$sp = array();
		$sp["clientName"] = WoogleConfig::get('remoteServiceName');
		$sp["clientVersion"] = WoogleConfig::get('remoteServiceVersion');
		$sp["host"] = getenv('HTTP_HOST');
		$sp["referer"] = $referer;
		$sp["remoteIP"] = getenv('REMOTE_ADDR');
		$sp["anonUserId"] = WoogleUtil::getAnonUserID();
		$sp["authKey"] = WoogleConfig::get('remoteAuthKey');
		$sp["group"] = WoogleConfig::getIndexGroup();
		WoogleUtil::debug('Group is set to ' . WoogleConfig::getIndexGroup());
		foreach ($sp as $k => $v) {
			if (!is_string($v)) {
				$sp[$k] = "";
			}
		}
		WoogleUtil::debug('Group is set to ' . $sp["group"]);
		WoogleUtil::profileOut(__METHOD__);
		return $sp;
	}
	
	
}
