<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/*
 * Matches a new/changed article against watches on Woogle: Search pages
 * Similar to services such as "Google Alerts" watchers are notified about
 * these results
 */
class WoogleAlertManager {
	
	// constants
	public static $STATUS_CREATED 	= 1;
	public static $STATUS_UPDATED	= 2;
	
	/**
	 * Method called in WoogleHooks.php when page is updated/created
	 * 
	 * @param $article			WoogleDocument object
	 * @param $status			one if the static classvariables
	 * @return void
	 */
	public static function matchDocument($wd, $status){		
		WoogleUtil::profileIn(__FUNCTION__);
		
		// TODO leverage status parameter
		
		// get all watches on woogle main
		$w = WoogleMWUtil::getWatches(WOO_NS_MAIN);
		
		$userAlerts = self::getUserAlerts( $w , $wd);

		if( $userAlerts ) {
			self::notifyUsers( $userAlerts , $wd);	
		}
				
		WoogleUtil::profileOut(__FUNCTION__);
	}
	
	private static function stripWildcards($query){
		$query = str_replace('*', '', $query);
		$query = str_replace('?', '', $query);
		return $query;  
	}	
	
	/**
	 * 
	 * @param 		two-dimensional array of page titles/user Ids (['Page_Title'][1]) $w
	 * @param 		WoogleDocument $wd
	 * @return		two-dimensional array of user id/page titles ([1]['Page_Title']
	 */
	public static function getUserAlerts( $w , $wd) {
		WoogleUtil::profileIn(__FUNCTION__);
				
		// get article content
		$content 	= $wd->getContent();
		$title		= $wd->getLabel();
									
		// TODO WoogleMWUtil::stripMediaWikiMarkup()
		
		$users = array();

		// get the Keys for the watches (i.e. 'Deut*' for an Watch on Woogle:Deut*)
		$k = array_keys($w);
		
		// iterate all page titles
		foreach($k as $key){			
			// save the key without Wildcards in a variable for search in text and Title
			$key_stripped = self::stripWildcards($key);
	
			// do naive initial implementation
			// if watch either matches title or content
			if ((strpos($title, $key_stripped) !== false) || (strpos($content, $key_stripped) !== false)){

				// iterate users watching title
				foreach($w[$key] as $u){
					$users[$u][] = $key; // add matching key					
				}
			}			
		}
		
		// TODO ?
		if( $users) {
			WoogleUtil::profileOut(__FUNCTION__);
			return $users;
		} else {
			WoogleUtil::profileOut(__FUNCTION__);
			return null;
		}
		
	}
	
	/**
	 * Send out E-Mail notifications to users watching a query 
	 * 
	 * @param $userAlerts - Array of users which have to be alerted
	 * @param $wd - WoogleDocument Object
	 * @return none
	 */
	private static function notifyUsers( $users, $wd ) {
		global $wgSitename, $wgContLang;
		WoogleUtil::profileIn(__FUNCTION__);
		
		$uk 			= array_keys($users);
		$title 			= $wd->getLabel();
		$link			= $wd->getUri();
		$lastChanged 	= $wd->getChanged();
		
		// $lastChanged = WoogleUtil::getDateTime( $lastChanged );		
		
		foreach($uk as $ukey) {
			
			// Get the UserObject by the given ID
			$user = User::newFromId( $ukey );			

			$timecorrection = $user->getOption( 'timecorrection' );
							
			$format = $user->getDatePreference();
			$time = $wgContLang->timeanddate( $lastChanged, true, $format, $timecorrection);
			
			$i = 0;								
			$matches = '';				
			foreach($users[$ukey] as $match){					
				if( $i == 0 ) {
					$matches = $match;					 
					// Add slashes for all further search-Results
				} else {					
					$matches .= ' / '.$match;
				}		
				$i++;					
			}		

			// Create the localized Email subject
			$subject = WoogleUtil::getMsg('WoogleAlertNotificationSubject', $wgSitename );

			if( $user->getRealName() ) {
				$username = $user->getRealName();
			} else {
				$username = $user->getName();
			}			
			// Create the localized Email body
			
			$body = WoogleUtil::getMsg('WoogleAlertNotificationBody', 
										$username , $title ,	$wgSitename, $time, $matches, $link );
										
			// If $debug ist true, no email will be send. 
			$debug = false; 
						
			if( $debug) {
				WoogleUtil::info('Editor: '	. $editor);
				WoogleUtil::info('Subject: '. $subject);
				WoogleUtil::info('Body: '	. $body);
			} else {
				// Check if an Email address is given
				if( $user->getEmail() ) {
					// Send an E-Mail to the User				
					$user->sendMail($subject , $body, $wgSitename);
				}
			}
		}
		WoogleUtil::profileOut(__FUNCTION__);
	}
	

}
