<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 * This is a special page checking if files attached in the Wiki can
 * be indexed properly (i.e. content can be extracted)
 */
class WoogleSpecialCheckFiles extends SpecialPage {
	
	function __construct() {
		WoogleUtil::profileIn(__METHOD__);
		parent::__construct(WoogleUtil::getSpecialPageLocal('CheckFiles'), '', false);
		WoogleUtil::profileOut(__METHOD__);
	}

	// Here the inline output of the Special page will be created
	function execute($par) {
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);

		// show title
		$this->setHeaders();
		$this->returntitle = Title::makeTitle(NS_SPECIAL, WoogleUtil::getSpecialPageLocal('CheckFiles'));
		
		// Intro
//		$wgOut->addHTML(WoogleUtil::getMsg('HelpIntro'));

		// TODO this is duplicated code from WoogleIndexService
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('image'));

		$sql = 'SELECT img_name as name, img_major_mime as major, img_minor_mime as mime, img_size as size, img_timestamp as timestamp, ' .
		       'img_sha1 as sha1 FROM ' . $image;
		$res = $db->doQuery($sql);
		
		$fDocs = array();
		while ($row = $db->fetchObject($res)) {
			$filename = $row->name;

			// get local file path
			$ftitle = Title::makeTitleSafe( NS_FILE, $filename );
			$lfile = wfLocalFile( $ftitle );
			$localFilePath = $lfile->getPath();

			// get URI
			$uri = $lfile->getURL();
			$uri = $wgServer . $uri;

      		// get mime type
      		$major 	= $row->major;
			$mime 	= $row->mime;

     		// using the sha1 value as ID as there seems to be no other id for files
			$id = $row->sha1;

			if ( $lfile && $lfile->exists() && file_exists($localFilePath) ) {

				// deal only with PDF for the moment
				if ( $mime == 'pdf' ) {
					$text = WooglePDFExtractor::pdfFile2string($localFilePath);
					if ($text == '') WoogleUtil::error('Could not extract content from PDF file: ' . $localFilePath);

				}

			} else {
				WoogleUtil::error('File ' . $localFilePath . ' can not be indexed.');
			}
		}
		
		
		
		
				
		WoogleUtil::profileOut(__METHOD__);
	}
	
}
