<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

class WoogleSpecialFeedback extends SpecialPage {
	
	protected $_name 	= '';
	protected $_email 	= '';
	protected $_text 	= '';
	protected $_subject = '';
	
	public function __construct() {
		WoogleUtil::profileIn(__METHOD__);
		parent::__construct(WoogleUtil::getSpecialPageLocal('Feedback'), '', false);
		WoogleUtil::profileOut(__METHOD__);
	}

	// Here the inline output of the Special page will be created
	function execute($par) {
		global $wgOut, $wgUser, $wgRequest;
		WoogleUtil::profileIn(__METHOD__);
		
		$this->setHeaders();
		$returntitle = Title::makeTitle(NS_SPECIAL, WoogleUtil::getSpecialPageLocal('Feedback'));
		
		$errors = array();
		
		$wgOut->addWikiText( WoogleUtil::getMsg("SpecialPageFeedbackMeta"));
		
		// if par == 'send' , then validate the user input
		if( $par == 'send' ) {
			
			if ( !$wgUser->matchEditToken( $wgRequest->getVal( 'token' ) ) ) {
				$errors['general'] = WoogleUtil::getMsg("UserTokenError");
			}
			
			if(empty($_POST['name'])) {
				$errors['name'] = WoogleUtil::getMsg("SpecialPageFeedbackNameForgotten");
			} else {
				$this->_name = htmlspecialchars( $_POST['name']);	
			}

			if (isset($_POST['email'])) $this->_email = htmlspecialchars($_POST['email']);
			if(empty($_POST['email']) || !WoogleMWUtil::isEmailValid($_POST['email'])) {
				$errors['email'] = WoogleUtil::getMsg("SpecialPageFeedbackEmailForgotten");
			}

			if (isset($_POST['subject'])) $this->_subject = htmlspecialchars($_POST['subject']);
			if ( (empty($_POST['subject']) || (strlen($_POST['subject']) < 5))){
				$errors['subject'] = WoogleUtil::getMsg("SpecialPageFeedbackSubjectForgotten");
			} 

			if (isset($_POST['text'])) $this->_text = htmlspecialchars($_POST['text']);
			if( (empty($_POST['text'])) || (strlen($_POST['text']) < 10 )) {
				$errors['text'] = WoogleUtil::getMsg("SpecialPageFeedbackTextForgotten");
			} 
			
			if( sizeof($errors) == 0 ) {
				$wgOut->addHTML( WoogleUtil::getMsg("SpecialPageFeedbackSend") );
				WoogleUtil::profileOut(__METHOD__);
				$this->sendEmail();

				// write log				
				WoogleUtil::feedback($this->_name.' , '.$this->_email.' . '.$this->_subject.' , '.$this->_text);									
				return;
			}
		}
		
		global $wgUser;
		if (!$wgUser->isAnon()){
			$this->_name = 	$wgUser->getRealName();
			if ($this->_name == '') $this->_name = $wgUser->getName();
			$this->_email = $wgUser->getEmail();
		} 
		
		include WoogleUtil::getTplFolder().'feedback_form.php';
		
		$wgOut->addHTML( $html);
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	private function sendEmail(){
		
		$senderMail 	= $this->_email;
		$senderName 	= $this->_name;		
		$subject 		= $this->_subject;
		$mailtext 		= $this->_text;
		$link 			= WoogleMWUtil::getArticleAbsolutePath();
		
		$bodytext = $mailtext."\n\n from ".$senderName."\n Email: ".$senderMail."\n Sended from: ".$link;	
		
		// return nl2br($subject.' '.$bodytext);
		WoogleUtil::sendFeedbackMail($subject, $bodytext);
	}
	
}
