<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 * This is the dynamic end user help page of Woogle
 * 
 * It displays configuration-specific help text, depending on the
 * global Woogle configuration settings
 * 
 */
class WoogleSpecialHelp extends SpecialPage {
	
	function __construct() {
		WoogleUtil::profileIn(__METHOD__);
		parent::__construct(WoogleUtil::getSpecialPageLocal('Help'), '', false);
		WoogleUtil::profileOut(__METHOD__);
	}

	// Here the inline output of the Special page will be created
	function execute($par) {
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);

		// show title
		$this->setHeaders();
		$this->returntitle = Title::makeTitle(NS_SPECIAL, WoogleUtil::getSpecialPageLocal('Help'));
		
		// Intro
		$wgOut->addHTML(WoogleUtil::getMsg('HelpIntro'));
		
		//
		// 	Check if Woogle is enabled etc.
		// 
		if (!WoogleConfig::get('core')){
			$wgOut->addHTML(WoogleUtil::getMsg('HelpDisabledSystem'));
			return;
		}
		
		if (!WoogleConfig::hasPermission()){
			$wgOut->addHTML(WoogleUtil::getMsg('HelpPermitted'));
			return;
		}
		
		if (!WoogleConfig::getPref('General', 'UseWoogle')){
			$wgOut->addHTML('<div style="text-align: center; background: #F7F7F7; border: 4px solid #B0BFE8; padding: 0.5em; clear: both; ">');
			$wgOut->addWikiText(WoogleUtil::getMsg('HelpDisabledUser'));
			$wgOut->addHTML('</div>');
		}
		
		//
		//	Show help page 
		//
		
		// hook for injecting HTML in instrumentation mode
		wfRunHooks('WoogleHelpInit', array($wgOut));	
		
		//
		// Search
		//
		$wgOut->addHTML('<h2>' . WoogleUtil::getMsg('HelpSearch') . '</h2><a name="help_search"></a>');
	
		// Via (SpecialSearch, MW-Box, OpenSearch
		if (WoogleConfig::get('replace')){
			$wgOut->addHTML(WoogleUtil::getMsg('HelpSearchReplace'));
		}
		
		$wgOut->addWikiText(WoogleUtil::getMsg('HelpSearchText'));
		
		if (!WoogleUtil::getQueryService()->isNative()){
			$wgOut->addHTML(WoogleUtil::getMsg('HelpSearchAdvanced'));
			$wgOut->addHTML($this->getThumb('help_search_ui_advanced', 150, WoogleUtil::getMsg('HelpSearch')));
		} else {
			$wgOut->addHTML($this->getThumb('help_search_ui', 150, WoogleUtil::getMsg('HelpSearch')));
		}
		
		// Woogle pages
		if (WoogleConfig::get('advanced')){
			$wgOut->addHTML($this->getThumb('help_search_results', 150, WoogleUtil::getMsg('HelpResultsCaption')));
			$wgOut->addHTML('<h3>' . WoogleUtil::getMsg('HelpWooglePages') . '</h3><a name="help_wooglepages"></a>');
			$wgOut->addHTML(WoogleUtil::getMsg('HelpWooglePagesText'));
		}

		// Autocomplete
		if (WoogleConfig::get('autoComplete')){
			$wgOut->addHTML('<h3>' . WoogleUtil::getMsg('HelpAutocomplete') . '</h3><a name="help_autocomplete"></a>');
			$wgOut->addHTML($this->getThumb('help_autocomplete', 150, WoogleUtil::getMsg('HelpAutocomplete')));
			$wgOut->addHTML(WoogleUtil::getMsg('HelpAutocompleteText'));
			if (WoogleConfig::get('restrictAutoComplete')){
				$wgOut->addHTML(WoogleUtil::getMsg('HelpAutocompleteRestricted'));
			} else {
				$wgOut->addHTML(WoogleUtil::getMsg('HelpAutocompleteFull'));
			}
		}
		
		// OpenSearch
		$wgOut->addHTML('<h3>' . WoogleUtil::getMsg('HelpOpenSearch') . '</h3><a name="help_opensearch"></a>');
		$wgOut->addHTML($this->getThumb('help_open_search_add', 150, WoogleUtil::getMsg('HelpOpenSearch')));
		$wgOut->addHTML(WoogleUtil::getMsg('HelpOpenSearchText'));
		if (!WoogleUtil::getQueryService()->isNative()){
			$wgOut->addHTML(WoogleUtil::getMsg('HelpOpenSearchRemote'));
		}
		
		// Syntax
		$wgOut->addHTML('<h3>' . WoogleUtil::getMsg('HelpSyntax') . '</h3><a name="help_syntax"></a>');
		$wgOut->addWikiText(WoogleUtil::getMsg('HelpSyntaxText'));
		
		// Social ranking
		if (WoogleConfig::get('socialResults')){
			$wgOut->addHTML('<h3>' . WoogleUtil::getMsg('HelpSyntax') . '</h3><a name="help_syntax"></a>');
			$wgOut->addHTML(WoogleUtil::getMsg('HelpSocialResults'));
		}
		
		//
		// Alerts
		//
		if (WoogleConfig::get('watchSearchResults')){
			$wgOut->addHTML('<h2>' . WoogleUtil::getMsg('HelpAlerts') . '</h2><a name="help_alerts"></a>');
			$wgOut->addWikiText(WoogleUtil::getMsg('HelpAlertsText'));
		}
		
		//
		// Knowledge sharing
		//
		$wgOut->addHTML('<h2>' . WoogleUtil::getMsg('HelpKnowledgeSharing') . '</h2><a name="help_kns"></a>');
		
		// Red links
		if (WoogleConfig::get('redLinkInfo')){
			$wgOut->addHTML($this->getThumb('help_red_link_asterisk', 150, WoogleUtil::getMsg('HelpRedCaption')));
			$wgOut->addHTML(WoogleUtil::getMsg('HelpKnowledgeSharingRed'));
		}
		
		// Stats
		$wgOut->addWikiText(WoogleUtil::getMsg('HelpKnowledgeSharingStats'));
		
		//
		// Configuration
		//
		$wgOut->addHTML('<h2>' . WoogleUtil::getMsg('HelpConfiguration') . '</h2><a name="help_config"></a>');
		$wgOut->addHTML($this->getThumb('help_preferences', 150, WoogleUtil::getMsg('HelpConfiguration')));
		$wgOut->addWikiText(WoogleUtil::getMsg('HelpConfigurationText'));
		
		//
		// Privacy
		//
		$wgOut->addHTML('<h2>' . WoogleUtil::getMsg('HelpPrivacy') . '</h2><a name="help_privacy"></a>');
		$wgOut->addHTML(WoogleUtil::getMsg('HelpPrivacyText'));
		
		//
		// Evaluation
		//
		if (WoogleConfig::get('instrumentation')){
			$wgOut->addHTML('<h2>' . WoogleUtil::getMsg('HelpEvaluation') . '</h2><a name="help_eval"></a>');
			//$wgOut->addHTML($this->getThumb('help_config', 350));
			$wgOut->addWikiText( WoogleUtil::getMsg('HelpEvaluationText'));
		}
		
		//
		// Feedback
		//
		$wgOut->addHTML('<h2>' . WoogleUtil::getMsg('HelpFeedback') . '</h2><a name="help_feedback"></a>');
		$wgOut->addWikiText(WoogleUtil::getMsg('HelpFeedbackText'));

		$wgOut->addHTML('<h2>' . WoogleUtil::getMsg('HelpAbout') . '</h2><a name="help_about"></a>');
		$wgOut->addHTML(WoogleUtil::getMsg('HelpAboutText'));
		
		
		// hook for injecting HTML in instrumentation mode
		wfRunHooks('WoogleHelpExit', array($wgOut));
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Helper function to create MediaWiki-Style thumbnail box for screenshots
	 * 
	 * @param String	$imgname - filename of the image		
	 * @param int		$width - thumbnail box width
	 * @param String	$caption - thumbnail caption (optional)
	 */
	private function getThumb($imgname, $width, $caption=''){
		$res = '<div class="thumb tright"><div class="thumbinner" style="width:' . ($width + 2) . 'px;">
					<a href="' . WoogleUtil::getImagePath() . 'screenshots/' . $imgname . '.png" class="image" title="' . $caption . '">
					<img alt="" src="' . WoogleUtil::getImagePath() . 'screenshots/' . $imgname . '.png" width="' . $width . '" border="0" class="thumbimage" /></a>
					<div class="thumbcaption">' . $caption . '</div></div></div>';
		return $res;
	}
}
