<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 * This class reprecents the Special:Woogle special page which disyplays
 * a raw Woogle search form
 * 
 * The display of the form and the results is handled by @see WoogleSearch.php
 * 
 */

class WoogleSpecialSearch extends SpecialPage{
	
	/*
	 * Constructor
	 */
	function __construct() {
		WoogleUtil::profileIn(__METHOD__);

		// call partent constructor with parameters: title, required permissions,
		// and if page should be listed on Special:SpecialPages
		parent::__construct(WoogleUtil::getSpecialPageLocal('Search'), '', false);
				
		WoogleUtil::profileOut(__METHOD__);		
	}

	/*
	 *  Here the inline output of the Special page will be created
	 */
	function execute($par) {
		global $wgRequest, $wgOut, $wgUsePathInfo;
		WoogleUtil::profileIn(__METHOD__);
						
		// get request parameters
		$searchString 	= $wgRequest->getVal('searchString');
		if (empty($par) && !empty($searchString)){

			// extract trail
			$requestUrl		= $wgRequest->getRequestURL();
			$requestUrl 	= explode('?', $requestUrl);
			$trail 			= WoogleUtil::extractField($requestUrl[1], 'searchString');

			// if we have index.php?title=... mode, $trail look a bit different
			if ($wgUsePathInfo === false) {
				$trail	= WoogleUtil::extractField($trail, 'title');
				$trail  = !empty($trail) ? '&' . $trail : '';
			} else {
				$trail 	= !empty($trail) ? '?' . $trail : '';
			}

			// redirect searchString to $par
			WoogleUtil::debug('WoogleSpecialSearch: Redirect');
			$wgOut->redirect(WoogleUtil::getWoogleLinkPrefixNonadvanced() . $searchString . $trail);
			WoogleUtil::profileOut(__METHOD__);
			return;
		}
		
		// create and call Woogle search object
		$result = new WoogleSearch(false);
		$result->setSearchString($par);
		$result->show();
		
		WoogleUtil::profileOut(__METHOD__);
	}
}
