<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

class WoogleSpecialStatistics extends SpecialPage {
	function __construct() {
		WoogleUtil::profileIn(__METHOD__);
		parent::__construct(WoogleUtil::getSpecialPageLocal('Statistics'), 'userrights', true);
		WoogleUtil::profileOut(__METHOD__);
	}

	// Here the inline output of the Special page will be created
	function execute($par) {
		global $wgOut, $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		
		// show title
		$this->setHeaders();
		$this->returntitle = Title::makeTitle(NS_SPECIAL, WoogleUtil::getSpecialPageLocal('Statistics'));
		
		if (!$this->userCanExecute($wgUser)) {
			$this->displayRestrictionError();
			return;
		}
		
		$qm = WoogleUtil::getQueryManager();
		
		// TODO localize
		
		//
		//	General
		//		
		$wgOut->addHTML('<h2>' .  WoogleUtil::getMsg('StatisticsGeneral') . '</h2>');
		
		$wgOut->addHTML('<h3>' .  WoogleUtil::getMsg('StatisticsPopularQueries') . '</h3>');
		$pop = $qm->getPopularQueries(10);
		$n=1;
		foreach ($pop as $pq) {
			$wgOut->addWikiText($n . '. ' . htmlspecialchars($pq->getName()) . ' (' . $pq->getFreq() . ')' );
			$n++;
		}
		$wgOut->addHTML('<br />');
		
		$wgOut->addHTML('<h3>' .  WoogleUtil::getMsg('StatisticsPopularResults') . '</h3>');
		$prs = $qm->getPopularResults(10);
		$n=1;
		foreach ($prs as $ef) {
			$e = $ef->getEntity();
			$wgOut->addWikiText($n . '. [' . $e['uri'] . ' ' . htmlspecialchars(urldecode($e['uri'])) . '] (' . $ef->getFreq() . ')' );
			$n++;
		}
		$wgOut->addHTML('<br />');
		
		// Only for WoogleRemote
		if (!WoogleUtil::getQueryService()->isNative()){
			$wgOut->addHTML('<h3>' .  WoogleUtil::getMsg('StatisticsPopularSources') . '</h3>');
			$prs = $qm->getPopularSources(10);
			$n=1;
			foreach ($prs as $ef) {
				$e = $ef->getEntity();
				$wgOut->addWikiText($n . '. ' .  WoogleUtil::getMsg('StatisticsRepoNumber') . ' ' . $e['repoId'] . ' (' . $ef->getFreq() . ')' );
				$n++;
			}
			$wgOut->addHTML('<br />');
		}

		
		//
		//	Unsatisfied
		//		
		$wgOut->addHTML('<h2>' .  WoogleUtil::getMsg('StatisticsUnsatisfiedNeeds') . '</h2>');
		$wgOut->addHTML('<h3>' .  WoogleUtil::getMsg('StatisticsUnsatisfiedFewClicks') . '</h3>');
		$qs = $qm->getUnsatisfiedQueriesWithFewHits(10);
		$n=1;
		foreach ($qs as $q) {
			$wgOut->addWikiText($n . '. ' . htmlspecialchars($q['needString']));		// TODO extend
			$n++;
		}
		$wgOut->addHTML('<br />');
		
		$wgOut->addHTML('<h3>' .  WoogleUtil::getMsg('StatisticsUnsatisfiedDiffClicks') . '</h3>');
		$qs = $qm->getUnsatisfiedQueriesWithManyDifferentClicks(10);
		$n=1;
		foreach ($qs as $q) {
			$wgOut->addWikiText($n . '. ' . htmlspecialchars($q['needString']));		// TODO extend
			$n++;
		}
		$wgOut->addHTML('<br />');
		
		// Only for WoogleNative
		if (WoogleUtil::getQueryService()->isNative()){
			$db = WoogleMWUtil::getDB();
			extract($db->tableNames('woogle_aggquery'));
			$wgOut->addHTML('<h3>' .  WoogleUtil::getMsg('StatisticsLessHits') . '</h3>');
			$this->showTable($db, 
				'SELECT aggquery_needexec AS count, aggquery_needstring AS query ' .
				'FROM ' . $woogle_aggquery . ' ' .
				'WHERE aggquery_resulthits < 10 ' .
				'ORDER BY aggquery_needexec DESC ' .
				'LIMIT 10;'
			);
			$wgOut->addHTML('<br />');
			
		}
		
		$wgOut->addHTML('<h3>' .  WoogleUtil::getMsg('StatisticsUnsatisfiedBrowse') . '</h3>');
		$qs = $qm->getUnsatisfiedQueriesWithMuchBrowsing(10);		
		$n=1;
		foreach ($qs as $q) {
			$wgOut->addWikiText($n . '. ' . htmlspecialchars($q['needString']));		// TODO extend
			$n++;
		}
		$wgOut->addHTML('<br />');
		
		//
		//	Woogle usage
		//		
		$wgOut->addHTML('<h2>' .  WoogleUtil::getMsg('StatisticsWoogleUsage') . '</h2>');
		$wgOut->addHTML('tbd.');
		
		//
		//	Old Stuff
		//		
		
		/*
		// show only local stats for WoogleNative currently
		if (WoogleUtil::getQueryService()->isNative()){
			
			/ QueryDb
			aggquery_id
			aggquery_qid
			aggquery_needstring
			aggquery_needexec
			aggquery_needexecclick
			aggquery_recency
			aggquery_firstdate
			aggquery_lastdate
			aggquery_resulthits
			aggquery_sizequeryusers
			aggquery_needexecpages
			aggquery_needexecavgpage
			aggquery_resultclicks
			aggquery_uniqueresults
			aggquery_queryusers
			aggquery_publicusers
			/
			
			$db = WoogleMWUtil::getDB();
			extract($db->tableNames('woogle_aggquery', 'woogle_aggclick'));
			
			$wgOut->addHTML('<h2>Top 10 Search Queries</h2>');
			$this->showTable($db, 
				'SELECT aggquery_needexec AS count, aggquery_needstring AS query ' .
				'FROM ' . $woogle_aggquery . ' ' .
				'ORDER BY aggquery_needexec DESC ' .
				'LIMIT 10;'
			);
			
			/* ClickDb 
			aggclick_id
			aggclick_qid
			aggclick_subject
			aggclick_clicks
			aggclick_position
			aggclick_recency
			aggclick_firstdate
			aggclick_lastdate
			aggclick_repoid 
			*/
			
			/*
			$wgOut->addHTML('<br /><br /><h2>Top 10 Search Results clicked</h2>');
			$this->showTable($db, 
				'SELECT aggclick_clicks AS count, aggclick_qid AS query ' .
				'FROM ' . $woogle_aggclick . ' ' .
				'ORDER BY aggclick_clicks DESC ' .
				'LIMIT 10;'
			);/
			
			$wgOut->addHTML('<br /><br /><h2>Top 10 Search Results clicked</h2>');
			$this->showTable($db, 
				'SELECT aggclick_clicks AS count, aggclick_subject AS query ' .
				'FROM ' . $woogle_aggclick . ' ' .
				'GROUP BY aggclick_qid ' .
				'ORDER BY aggclick_clicks DESC ' .
				'LIMIT 10;'
			);
		}
		*/
		
		WoogleUtil::profileOut(__METHOD__);
		
	}
	private function showTable($db, $sql) {
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		$res = $db->doQuery($sql);
		//$wgOut->addHTML('<table>');
		//$wgOut->addHTML('<tr><th>#</th><th>Query</th></tr>');
		$n=1;
		while ($row = $db->fetchObject($res)) {
			//$wgOut->addHTML('<tr><td>' . $row->count . '</td><td>' . $row->query . '</td></tr>');
			$wgOut->addWikiText($n . '. ' . htmlspecialchars($row->query) . ' (' .  $row->count . ')');
			$n++;
		}
		$wgOut->addHTML('</table>');
		WoogleUtil::profileOut(__METHOD__);
	}
}
