<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

require_once('Woogle_TestCase.php');

class WoogleIndexServiceNativeTest extends Woogle_TestCase{
	

    protected function setUp(){
    	$this->prepareTestDb();
		$this->prepareTestIndex();
	}
	
	/* Test methods */
	
	public function testCheckIndexService(){
		
		// wrong
		$tmp = WoogleLucene::getIndexPath();
		WoogleLucene::setIndexPath('Z:\_wrongPath');
		$this->assertFalse($this->is->checkIndexService());
		WoogleLucene::setIndexPath($tmp);
		
		// current
		$this->assertTrue($this->is->checkIndexService());
		
	}
	
	public function testGetIndexFileSize(){
		echo $this->is->getIndexFileSize();
		// TODO
		//$this->assertTrue($this->is->getIndexFileSize() < 0.1);
	}
	
	public function testCommitIndex(){
		// TODO
	}
	
	public function testDoAddItems(){
		// TODO
	}
	
	public function testDoUpdateItem(){
		// TODO
	}
	
	// this also covers exists(), doAddItem() and doDeleteItem()
	public function testNumDocs(){
		
		$this->assertEquals($this->num_test_docs, $this->is->numDocs());
		
		$wd = new WoogleDocument();
		$wd->setId(WoogleQueryServiceCommon::getId('999'));
		$wd->setShortId('999');
		$wd->setLabel('Page999');
		$wd->setContent('This is content');
		$wd->setUri('http://localhost/wiki/Page999');
		
		// add
		$this->is->doAddItem($wd, true);
		$this->is->commitIndex();
		$this->assertEquals($this->num_test_docs+1, $this->is->numDocs());
		
		// remove
		$this->assertTrue($this->is->exists($wd->getShortId()));
		$this->is->doDeleteItem($wd->getShortId());
		$this->is->commitIndex();
		WoogleLucene::optimizeIndex();	// otherwise, exists will return true, although document marked deleted
		$this->assertFalse($this->is->exists($wd->getShortId()));
		$this->assertEquals($this->num_test_docs, $this->is->numDocs());

	}

	public function testIndexAll(){
		
		$this->assertEquals($this->num_test_docs, $this->is->numDocs());
		//$this->assertEquals(0, $this->is->numDocs());
		$this->is->index(true); // index all
		$this->assertEquals($this->num_test_docs, $this->is->numDocs());

		// exclude namespaces
		WoogleConfig::set('excludeNamespaces', array(NS_MEDIAWIKI, NS_TALK));
		$this->is->index(true); // index all
		$this->assertEquals($this->num_test_docs-2, $this->is->numDocs());
		
		
		// TODO add imagedata
	}
	


	
	
}



?>
