<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

require_once('Woogle_TestCase.php');

/*
global $wgUser;
$wgUser = User::newFromName('WikiSyop');
*/

class WoogleQueryServiceNativeTest extends Woogle_TestCase{
	
    protected function setUp(){
		$this->prepareTestDb();
    	$this->prepareTestIndex();
		$this->prepareQueryService();		
		
		$this->qm = WoogleUtil::getQueryManager();
   	  	$this->prepareQueryTestData();
    }
	
	protected function tearDown(){
		$this->removeQueryTestData();
		unset($this->qm);
		unset($this->qs);
	}
	
	/* Test methods */
	
	public function testCheckQueryService(){
		$this->assertEquals($this->num_test_docs, $this->qs->checkQueryService());
	}
	
	public function testIsNative(){
		$this->assertTrue($this->qs->isNative());
	}
	
	public function testGetAutocomplete(){
		$this->assertEquals($this->num_test_docs, $this->is->numDocs());

		$r = $this->qs->getAutocomplete('unr', 10);
		$this->assertEquals(1, count($r));
		$this->assertEquals('unrelated', $r[0]['entity']);
		$this->assertEquals(1, $r[0]['freq']);
		
		$r = $this->qs->getAutocomplete('über', 10);
		$this->assertEquals(2, count($r));
		
		$r = $this->qs->getAutocomplete('Über', 10);
		$this->assertEquals(2, count($r));

		$r = $this->qs->getAutocomplete('ver', 10);
		$this->assertEquals(1, count($r));
		
	}

	public function testExecuteKeywordQueryValues(){
		$this->assertEquals($this->num_test_docs, $this->is->numDocs());
		
		$values = self::getValues(); // TODO leverage
		
		// plain query and variations
		$res = $this->qs->executeKeywordQueryValues('This', 10, 0, $values, 'unittest');
		$this->assertEquals(5, $res['numHits']); // 5 pages should match 'this'

		$res = $this->qs->executeKeywordQueryValues('this', 10, 0, $values, 'unittest');
		$this->assertEquals(5, $res['numHits']); // 5 pages should match 'this'

		$res = $this->qs->executeKeywordQueryValues('ThiS', 10, 0, $values, 'unittest');
		$this->assertEquals(5, $res['numHits']); // 5 pages should match 'this'

		// umlauts
		$res = $this->qs->executeKeywordQueryValues('veränderung', 10, 0, $values, 'unittest');
		$this->assertEquals(2, $res['numHits']);

		$res = $this->qs->executeKeywordQueryValues('überraschung', 10, 0, $values, 'unittest');
		$this->assertEquals(1, $res['numHits']);
		
		// wildcards
		$res = $this->qs->executeKeywordQueryValues('ver?nderung', 10, 0, $values, 'unittest');
		$this->assertEquals(2, $res['numHits']);
		
		$res = $this->qs->executeKeywordQueryValues('verände?ung', 10, 0, $values, 'unittest');
		$this->assertEquals(2, $res['numHits']);
		
		$res = $this->qs->executeKeywordQueryValues('über*', 10, 0, $values, 'unittest');
		$this->assertEquals(2, $res['numHits']);
				
		// fuzzy
		$res = $this->qs->executeKeywordQueryValues('unrelated', 10, 0, $values, 'unittest');
		$this->assertEquals(2, $res['numHits']);
		
		$res = $this->qs->executeKeywordQueryValues('unreluted', 10, 0, $values, 'unittest');
		$this->assertEquals(0, $res['numHits']);

		$res = $this->qs->executeKeywordQueryValues('unreluted~', 10, 0, $values, 'unittest');
		$this->assertEquals(2, $res['numHits']);
		
		// phrase
		$res = $this->qs->executeKeywordQueryValues('talk page', 10, 0, $values, 'unittest');
		$this->assertEquals(2, $res['numHits']);
		
		$res = $this->qs->executeKeywordQueryValues('"talk page"', 10, 0, $values, 'unittest');
		$this->assertEquals(1, $res['numHits']);
		
		// hha memorial test
		$res = $this->qs->executeKeywordQueryValues('heiko', 10, 0, $values, 'unittest');
		$this->assertEquals(1, $res['numHits']);
		
		$res = $this->qs->executeKeywordQueryValues('Heiko Haller/Chairing', 10, 0, $values, 'unittest');
		$this->assertEquals(1, $res['numHits']);
		
		
		// TODO add more	
		
	}
	
	public function testHighlight3(){
		
		$terms = array('test', 'karlsruhe', 'überall');
		
		$basetext = 'Dies ist ein kleiner Test in Bezug auf Mannheim.';
		$res = $this->qs->highlight3($basetext, $terms);
		$this->assertTrue(strpos($res, WoogleQueryService::$highlightStart) > -1);
		$this->assertTrue(strpos($res, WoogleQueryService::$highlightStop) > -1);
		
		// no match
		$basetext = 'Dies ist ein kleiner Versuch in Bezug auf Mannheim.';
		$res = $this->qs->highlight3($basetext, $terms);
		$this->assertEquals($basetext, $res);
		
		// umlaut
		$basetext = 'Dies ist ein kleiner Versuch in Bezug auf Mannheim - überalledem.';
		$res = $this->qs->highlight3($basetext, $terms);
		$this->assertTrue(strpos($res, WoogleQueryService::$highlightStart) > -1);
		$this->assertTrue(strpos($res, WoogleQueryService::$highlightStop) > -1);
		
		
		// TODO add umlaut etc.
		
		
	}

	// TODO: highlight, cut


	
}



?>
